"""
The pytest tool invokes pytest based tests.
"""
import logging

from menhir.tool import Tool
from menhir.tool_utils import (
    OK,
    FAIL,
    NOTHING_TO_DO,
    package_script,
    tool_env,
    working_dir,
)

log = logging.getLogger(__name__)


def tool():
    return PyTest()


class PyTest(Tool):

    def is_using_tool(tool, path, info):
        from os.path import exists, join
        path = join(path, 'setup.cfg')
        if exists(path):
            with open(path, "r") as file:
                data = file.read()
                return 'pytest' in data

    def dependencies(tool, path):
        return []

    def add_arg_parser(tool, parser):
        parser.description = "Invoke pytest based tests"
        parser.add_argument('args', nargs='*', help='pytest arguments')

    def execute_build_phase(tool, path, info, args,):
        """Execute a build phase."""
        import subprocess
        from menhir.project import project_name_for_dir

        log.info('pytest: %s %s %s', path, info, args)
        if info.get('changed') or info.get('changed_dependents') or args.all:
            log.info('Running pytest in %s', path)
            project_name = project_name_for_dir(path)
            env = tool_env()
            env['MENHIR_PROJECT'] = project_name
            with package_script("/tools/pytest/test.sh") as f:
                with working_dir(path):
                    res = subprocess.call([f.name] + args.args, env=env)
                    if res:
                        return FAIL
                    return OK
        else:
            return NOTHING_TO_DO



"""Tool interface for tool implementations.

A tool is implemented as an opaque class.

"""


def missing_impl(tool, f):
    raise Exception(
        'incorrect tool implementation',
        'Tool implementation %s is missing a "%s" implementation' % (
            type(tool),
            f,
        )
    )


class Tool(object):
    def __str__(self):
        return "<%s>" % type(self).__name__

    def __repr__(self):
        return "<%s>" % type(self).__name__

    def name(self):
        return type(self).__name__.lower()

    def is_using_tool(tool, path, info):
        """Predicate to test if the path contains a project using the tool."""
        missing_impl(tool, Tool.is_using_tool.__name__)

    def dependencies(tool, path):
        """Return a list of dependency prefixes for the project at path."""
        missing_impl(tool, Tool.dependencies.__name__)

    def execute_build_phase(tool, path, info, args):
        """Execute a build phase."""
        missing_impl(tool, Tool.execute_build_phase.__name__)

    def add_arg_parser(tool, parsers):
        """Add an arg parser for the tool to parsers."""
        missing_impl(tool, Tool.add_arg_parser.__name__)



"""
Pyenv based tool for menhir.

The ``pyenv`` tool provides the ``virtualenv`` target to create a
virtualenv for each of the repository's projects.  The virtualenv is
loaded with all the project's dependencies as specified in
``requirements*.txt``.
"""
import logging

from menhir.tool import Tool
from menhir.tool_utils import OK, FAIL, NOTHING_TO_DO, tool_env, working_dir

log = logging.getLogger(__name__)


def tool():
    return Pyenv()


class Pyenv(Tool):

    def is_using_tool(tool, path, info):
        from os.path import exists, join
        path = join(path, 'requirements.txt')
        return exists(path)

    def dependencies(tool, path):
        return []

    def add_arg_parser(tool, parser):
        parser.description = "Manage project specific pyenv virtualenvs"
        parsers = parser.add_subparsers(help="pyenv commands", dest='phase')
        parsers.add_parser(
            'virtualenv',
            help='Build a pyenv virtualenv with the projects dependencies'
        )

    def execute_build_phase(tool, path, info, args,):
        """Execute a build phase."""
        import subprocess
        from menhir.tool_utils import package_script
        from menhir.project import project_name_for_dir

        if args.phase == 'virtualenv':
            if (
                    args.all or
                    info.get('changed') or
                    info.get('changed_dependents') or
                    (info.get('changed_dependees') and
                     not args.only_if_changed)
            ):
                log.info('Running pyenv requirements in %s', path)

                env = tool_env()

                with package_script("/tools/pyenv/requirements.sh") as f:
                    with working_dir(path):
                        res = subprocess.call(
                            [f.name, path, project_name_for_dir(path)],
                            env=env,
                        )
                        if res:
                            return FAIL
                        return OK
            else:
                log.info('not running pyenv requirements in %s', path)
                return OK
        else:
            return NOTHING_TO_DO
